--|-----------------------------------------------------------------
--|-- G"unther K"ogel
--|-- University of Stuttgart
--|-----------------------------------------------------------------
-- error interface used by lalr

indexing
        date: "Tue Aug 11 16:55:01 MET DST 1992";
        revision: "1.1"

-- 1.1 new interface for WriteInfo
-- 1.0 initial part

class Errors
export
  ErrorMessage, ErrorMessageI,

-- ErrorClass
  Fatal, Restriction, Error, Warning, Repair, Note, Information,

-- ErrorCode
  NoText, SyntaxError, ExpectedTokens, RestartPoint, TokenInserted,
  WrongParseTable, OpenParseTable, ReadParseTable,

-- Parametertype
  None, IsInteger, IsShort, IsLong, IsReal, IsBoolean, IsCharacter,
  IsString

feature
  Fatal		: INTEGER is unique;
  Restriction	: INTEGER is unique;
  Error		: INTEGER is unique;
  Warning	: INTEGER is unique;
  Repair	: INTEGER is unique;
  Note		: INTEGER is unique;
  Information	: INTEGER is unique;

  NoText	: INTEGER is unique;
  SyntaxError	: INTEGER is unique;
  ExpectedTokens: INTEGER is unique;
  RestartPoint	: INTEGER is unique;
  TokenInserted	: INTEGER is unique;
  WrongParseTable: INTEGER is unique;
  OpenParseTable: INTEGER is unique;
  ReadParseTable: INTEGER is unique;

  None		: INTEGER is unique;
  IsInteger	: INTEGER is unique;
  IsShort	: INTEGER is unique;
  IsLong	: INTEGER is unique;
  IsReal	: INTEGER is unique;
  IsBoolean	: INTEGER is unique;
  IsCharacter	: INTEGER is unique;
  IsString	: INTEGER is unique;

	
ErrorMessage (
	ErrorCode,
	ErrorClass      : INTEGER;	-- CARDINAL
	Position_Line,
	Position_Column : INTEGER	-- tPosition
	) is
  local
    NIL : ANY;
  do
    ErrorMessageI (ErrorCode, ErrorClass, Position_Line, Position_Column, None, NIL);
  end; -- ErrorMessage

ErrorMessageI (
	ErrorCode,
	ErrorClass      : INTEGER;	-- CARDINAL
	Position_Line,
	Position_Column : INTEGER;	-- tPosition
	InfoClass       : INTEGER;	-- CARDINAL
	Info            : ANY		-- ADDRESS
	) is
  do
    WriteErrorMessage (ErrorCode, ErrorClass, Position_Line, Position_Column);
    WriteInfo (InfoClass, Info);
    io.error.new_line;
    if ErrorClass = Fatal then
      HALT (-ErrorCode);
    end;
  end; -- ErrorMessageI

WriteErrorMessage (
	ErrorCode,
	ErrorClass      : INTEGER;	-- CARDINAL
	Position_Line,
	Position_Column : INTEGER	-- tPosition
	) is
  do
    io.error.putint (Position_Line);
    io.error.putstring (", ");
    io.error.putint (Position_Column);
    io.error.putstring (": ");

    inspect ErrorClass
      when Fatal       then io.error.putstring ("Fatal       ");
      when Restriction then io.error.putstring ("Restriction ");
      when Error       then io.error.putstring ("Error       ");
      when Warning     then io.error.putstring ("Warning     ");
      when Repair      then io.error.putstring ("Repair      ");
      when Note        then io.error.putstring ("Note        ");
      when Information then io.error.putstring ("Information ");
      else                  io.error.putstring ("Error class: ");
                            io.error.putint    (ErrorClass);
    end; -- inspect

    inspect ErrorCode
      when NoText then 
      when SyntaxError     then io.error.putstring ("syntax error");
      when ExpectedTokens  then io.error.putstring ("expected tokens");
      when RestartPoint    then io.error.putstring ("restart point"	);
      when TokenInserted   then io.error.putstring ("token inserted ");
      when WrongParseTable then io.error.putstring ("parse table mismatch");
      when OpenParseTable  then io.error.putstring ("cannot open parse table");
      when ReadParseTable  then io.error.putstring ("cannot read parse table");
    else			io.error.putstring (" error code: ");
				io.error.putint    (ErrorCode);
    end;
  end; -- WriteErrorMessage

WriteInfo (
	InfoClass : INTEGER;	-- CARDINAL
	Info      : ANY		-- ADDRESS
	) is
  local
    ToInteger   : INT;		-- POINTER TO INTEGER;
    ToShort     : INT;		-- POINTER TO SHORTCARD;
    ToLong      : INT;		-- POINTER TO LONGINT;
    ToFloat     : FLOAT;	-- POINTER TO REAL;
    ToBoolean   : BOOL;		-- POINTER TO BOOLEAN;
    ToCharacter : CHAR;		-- POINTER TO CHAR;
    ToString    : STRING;	-- POINTER TO tString;
    ToSet       : SETS;		-- POINTER TO tSet;
    ToTokenSet  : SETS;		-- POINTER TO tSet;
    -- new
    i           : INTEGER;
    r           : REAL;
    c           : CHARACTER;
    b           : BOOLEAN;
  do
    if InfoClass /= None then
      io.error.putstring (": ");
      inspect InfoClass
        when IsInteger then
          ToInteger ?= Info;
          i := tointeger;
          io.error.putint (i);
        when IsShort then
          ToShort ?= Info;
          i := ToShort;
          io.error.putint (i);
        when IsLong then
          ToLong ?= Info;
          i := ToLong;
          io.error.putint (i);
        when IsReal then
          ToFloat ?= Info;
          r := ToFloat;
          io.error.putreal (r);
        when IsBoolean then
          ToBoolean ?= Info;
          b := ToBoolean;
          io.error.putbool (b);
        when IsCharacter then
          ToCharacter ?= Info;
          c := ToCharacter;
          io.error.putchar (c);
        when IsString then
          ToString ?= Info;
          if not ToString.void then
            io.error.putstring (ToString);
          end;
        else
          io.error.putstring ("info class: ");
          io.error.putint    (InfoClass);
	end;
      end; -- if
    end; -- WriteInfo

WriteToken (
	f    : FILE;	-- tFile
	Text : STRING	-- CARDINAL
	) is
  do
    f.putstring (Text);
  end; -- WriteToken

-- PROCEDURE WriteTokenSet (File: tFile; TokenSet: tSet);
-- not implemented
-- wiederspricht oo Ansatz

HALT ( exitcode : INTEGER ) is
  external
    exit language "C"
  do
    exit (exitcode);
  end; -- halt

end; -- Errors
